@@ -282,7 +282,7 @@ class SelectWinBase(gtk.ScrolledWindow):
 
 
 class ExerciseView(SelectWinBase):
-    max_exercise_label_width = int(gtk.gdk.Screen().get_width() * 0.90)
+    max_exercise_label_width = int(gtk.gdk.screen_width() * 0.90)
     def __init__(self):
         SelectWinBase.__init__(self)
         app = solfege.app
@@ -819,6 +819,8 @@ class MainWin(gtk.Window, cfg.ConfigUtils):
             box2.pack_start(gu.hig_label_widget(_("Solfege config file:"), gtk.Label(filesystem.rcfile()), sizegroup))
             box2.pack_start(gu.hig_label_widget(_("Solfege installation directory:"), gtk.Label(os.getcwdu()), sizegroup))
             box2.pack_start(gu.hig_label_widget(_("User manual in HTML format:"), gtk.Label(os.path.join(os.getcwdu(), "help")), sizegroup))
+            box2.pack_start(gu.hig_label_widget("gtk:", gtk.Label(str(gtk)), sizegroup))
+            box2.pack_start(gu.hig_label_widget("PYTHONHOME", gtk.Label(os.environ.get('PYTHONHOME', 'Not defined')), sizegroup))
             self.g_path_info_dlg.show_all()
             def f(*w):
                 self.g_path_info_dlg.hide()
